/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.tiff.persistence;

import java.util.Arrays;
import java.util.Date;
import it.tidalwave.image.metadata.TIFF;
import it.tidalwave.image.metadata.EXIFDirectoryGenerated.Contrast;
import it.tidalwave.metadata.persistence.spi.UnknownPropertyException;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class TIFFBeanAccessorTest 
  {
    private final static Date DATE1 = new Date(1,2,3,4,5,6);
    private final static Date DATE2 = new Date(2,3,4,5,6,7);
    
    private DirectoryBeanAccessor<TIFF> beanAccessor;
    private TIFF tiff;
    
    @Before
    public void setupFixture()
      {
        beanAccessor = new TIFFBeanAccessor();
        tiff = new TIFF();
        tiff.setArtist("Fabrizio Giudici");
        tiff.setContrast(Contrast.SOFT);
        tiff.setDateTimeAsDate(DATE1);
        tiff.setThumbnail("thumbnail".getBytes());
        tiff.setInterColourProfile("ICC".getBytes());
        tiff.setOriginalRawFileData("originalRaw".getBytes());
        tiff.setXMP("XMP".getBytes());
        tiff.setDNGPrivateData("DNGPrivate".getBytes());
        tiff.setUserComment("test".getBytes());
      }
    
    @After
    public void tearDownFixture()
      {
        beanAccessor = null;
        tiff = null;
      }
    
    @Test
    public void testIsPersistent()
      {
        for (final String propertyName : TIFFBasedBeanAccessor.TRANSIENT_PROPERTIES)
          {
            assertFalse(beanAccessor.isPersistent(propertyName));  
          }
      }
    
    @Test
    public void testGetPersistentPropertyNames()
      {
        assertEquals(Arrays.asList("#" + TIFF.DATETIME, "#" + TIFF.ARTIST, "#" + TIFF.CONTRAST, "dateTimeAsDate", "userCommentAsString"),
                beanAccessor.getPersistentPropertyNames(tiff));
      }
    
    @Test
    public void testGetPropertyType()
      throws UnknownPropertyException
      {
        assertEquals(String.class, beanAccessor.getDefaultPropertyType("#" + TIFF.ARTIST));
        assertEquals(Integer.class, beanAccessor.getDefaultPropertyType("#" + TIFF.CONTRAST)); // enum
        assertEquals(String.class, beanAccessor.getDefaultPropertyType("#" + TIFF.DATETIME));
        assertEquals(Date.class, beanAccessor.getDefaultPropertyType("dateTimeAsDate"));
      }
    
    @Test
    public void testConvertPropertyNameToPersistence()
      throws UnknownPropertyException
      {
        // TODO: it would be nice to test ALL properties...
        assertEquals("#" + TIFF.ARTIST, beanAccessor.convertPropertyNameToPersistence("artist"));
        assertEquals("#" + TIFF.CONTRAST, beanAccessor.convertPropertyNameToPersistence("contrast"));
        assertEquals("#" + TIFF.DATETIME, beanAccessor.convertPropertyNameToPersistence("dateTime"));
      }
    
    @Test
    public void testConvertPropertyNameFromPersistence()
      {
        // TODO: it would be nice to test ALL properties...
        assertEquals("artist", beanAccessor.convertPropertyNameFromPersistence("#" + TIFF.ARTIST));
        assertEquals("contrast", beanAccessor.convertPropertyNameFromPersistence("#" + TIFF.CONTRAST));
        assertEquals("dateTime", beanAccessor.convertPropertyNameFromPersistence("#" + TIFF.DATETIME));
        assertEquals("#99999", beanAccessor.convertPropertyNameFromPersistence("#99999")); // unknown tag
      }
    
    @Test
    public void testConvertValueToPersistence()
      throws UnknownPropertyException
      {
        assertEquals("Fabrizio Giudici", beanAccessor.convertValueToPersistence("artist", "Fabrizio Giudici"));
        assertEquals(1, beanAccessor.convertValueToPersistence("contrast", Contrast.SOFT));
      }
    
    @Test
    public void testConvertValueFromPersistence()
      throws UnknownPropertyException
      {
        assertEquals("Fabrizio Giudici", beanAccessor.convertValueFromPersistence("artist", "Fabrizio Giudici"));
        assertEquals(Contrast.SOFT, beanAccessor.convertValueFromPersistence("contrast", 1));
      }
    
    @Test
    public void testGetProperty()
      throws UnknownPropertyException
      {
        assertEquals("Fabrizio Giudici", beanAccessor.getProperty("#" + TIFF.ARTIST, tiff));
        assertEquals(1, beanAccessor.getProperty("#" + TIFF.CONTRAST, tiff));
        assertEquals("1901:03:03 04:05:06", beanAccessor.getProperty("#" + TIFF.DATETIME, tiff));
        assertEquals(DATE1, beanAccessor.getProperty("dateTimeAsDate", tiff));
      }
    
    @Test
    public void testSetProperty()
      throws UnknownPropertyException
      {
        beanAccessor.setProperty(tiff, "#" + TIFF.ARTIST, "John Doe");
        beanAccessor.setProperty(tiff, "#" + TIFF.CONTRAST, 2);
        beanAccessor.setProperty(tiff, "#" + TIFF.DATETIME, "1902:04:04 05:06:07");
        assertEquals("John Doe", tiff.getArtist());
        assertEquals(Contrast.HARD, tiff.getContrast());
        assertEquals(DATE2, tiff.getDateTimeAsDate());
        beanAccessor.setProperty(tiff, "dateTimeAsDate", DATE1);
        assertEquals(DATE1, tiff.getDateTimeAsDate());
      }
  }